@@ -24,6 +24,8 @@ extern "C" {
     RPY_EXPORTED
     int cppyy_compile(const char* code);
     RPY_EXPORTED
+    int cppyy_compile_silent(const char* code);
+    RPY_EXPORTED
     char* cppyy_to_string(cppyy_type_t klass, cppyy_object_t obj);
 
     /* name to opaque C++ scope representation -------------------------------- */
@@ -114,6 +116,8 @@ extern "C" {
     int cppyy_is_enum(const char* type_name);
     RPY_EXPORTED
     int cppyy_is_aggregate(cppyy_type_t type);
+    RPY_EXPORTED
+    int cppyy_is_default_constructable(cppyy_type_t type);
 
     RPY_EXPORTED
     const char** cppyy_get_all_cpp_names(cppyy_scope_t scope, size_t* count);
