@@ -55,6 +55,12 @@ typedef CPyCppyy::Parameter Parameter;
 // small number that allows use of stack for argument passing
 const int SMALL_ARGS_N = 8;
 
+// convention to pass flag for direct calls (similar to Python's vector calls)
+#define DIRECT_CALL ((size_t)1 << (8 * sizeof(size_t) - 1))
+static inline size_t CALL_NARGS(size_t nargs) {
+    return nargs & ~DIRECT_CALL;
+}
+
 // data for life time management ---------------------------------------------
 typedef std::vector<TClassRef> ClassRefs_t;
 static ClassRefs_t g_classrefs(1);
@@ -87,7 +93,7 @@ public:
     }
 
 public:
-    TInterpreter::CallFuncIFacePtr_t   fFaceptr;
+    TInterpreter::CallFuncIFacePtr_t fFaceptr;
     DeclId_t      fDecl;
     std::string   fName;
     TFunction*    fTF;
@@ -336,9 +342,9 @@ bool match_name(const std::string& tname, const std::string fname)
 
 
 // direct interpreter access -------------------------------------------------
-bool Cppyy::Compile(const std::string& code)
+bool Cppyy::Compile(const std::string& code, bool silent)
 {
-    return gInterpreter->Declare(code.c_str());
+    return gInterpreter->Declare(code.c_str(), silent);
 }
 
 std::string Cppyy::ToString(TCppType_t klass, TCppObject_t obj)
@@ -414,8 +420,14 @@ std::string Cppyy::ResolveName(const std::string& cppitem_name)
         return resolved;
     }
 
-// typedefs etc. (and cleanup STL classes for consistency)
+// typedefs etc. (and a couple of hacks around TClassEdit-isms, fixing of which
+// in ResolveTypedef itself is a TODO ...)
     tclean = TClassEdit::ResolveTypedef(tclean.c_str(), true);
+    pos = 0;
+    while ((pos = tclean.find("::::", pos)) != std::string::npos) {
+        tclean.replace(pos, 4, "::");
+        pos += 2;
+    }
     if (tclean.compare(0, 6, "const ") != 0)
         return TClassEdit::ShortType(tclean.c_str(), 2);
     return "const " + TClassEdit::ShortType(tclean.c_str(), 2);
@@ -660,7 +672,7 @@ bool Cppyy::IsComplete(const std::string& type_name)
 
     int oldEIL = gErrorIgnoreLevel;
     gErrorIgnoreLevel = 3000;
-    TClass* klass = TClass::GetClass(TClassEdit::ShortType(type_name.c_str(), 1).c_str());
+    TClass* klass = TClass::GetClass(type_name.c_str());
     if (klass && klass->GetClassInfo())     // works for normal case w/ dict
         b = gInterpreter->ClassInfo_IsLoaded(klass->GetClassInfo());
     else {    // special case for forward declared classes
@@ -715,7 +727,7 @@ void Cppyy::Destruct(TCppType_t type, TCppObject_t instance)
 
 
 // method/function dispatching -----------------------------------------------
-static TInterpreter::CallFuncIFacePtr_t GetCallFunc(Cppyy::TCppMethod_t method)
+static TInterpreter::CallFuncIFacePtr_t GetCallFunc(Cppyy::TCppMethod_t method, bool as_iface)
 {
 // TODO: method should be a callfunc, so that no mapping would be needed.
     CallWrapper* wrap = (CallWrapper*)method;
@@ -741,7 +753,7 @@ static TInterpreter::CallFuncIFacePtr_t GetCallFunc(Cppyy::TCppMethod_t method)
 // there is a different overload available that will do)
     auto oldErrLvl = gErrorIgnoreLevel;
     gErrorIgnoreLevel = kFatal;
-    wrap->fFaceptr = gInterpreter->CallFunc_IFacePtr(callf);
+    wrap->fFaceptr = gInterpreter->CallFunc_IFacePtr(callf, as_iface);
     gErrorIgnoreLevel = oldErrLvl;
 
     gInterpreter->CallFunc_Delete(callf);   // does not touch IFacePtr
@@ -778,26 +790,36 @@ void release_args(Parameter* args, size_t nargs) {
     }
 }
 
+static inline
+bool is_ready(CallWrapper* wrap, bool is_direct) {
+    return (!is_direct && wrap->fFaceptr.fGeneric) || (is_direct && wrap->fFaceptr.fDirect);
+}
+
 static inline
 bool WrapperCall(Cppyy::TCppMethod_t method, size_t nargs, void* args_, void* self, void* result)
 {
     Parameter* args = (Parameter*)args_;
+    bool is_direct = nargs & DIRECT_CALL;
+    nargs = CALL_NARGS(nargs);
 
     CallWrapper* wrap = (CallWrapper*)method;
-    const TInterpreter::CallFuncIFacePtr_t& faceptr = wrap->fFaceptr.fGeneric ? wrap->fFaceptr : GetCallFunc(method);
-    if (!faceptr.fGeneric)
+    const TInterpreter::CallFuncIFacePtr_t& faceptr = \
+        is_ready(wrap, is_direct) ? wrap->fFaceptr : GetCallFunc(method, !is_direct);
+    if (!is_ready(wrap, is_direct))
         return false;        // happens with compilation error
 
+    nargs = CALL_NARGS(nargs);
     if (faceptr.fKind == TInterpreter::CallFuncIFacePtr_t::kGeneric) {
         bool runRelease = false;
+        const auto& fgen = is_direct ? faceptr.fDirect : faceptr.fGeneric;
         if (nargs <= SMALL_ARGS_N) {
             void* smallbuf[SMALL_ARGS_N];
             if (nargs) runRelease = copy_args(args, nargs, smallbuf);
-            faceptr.fGeneric(self, (int)nargs, smallbuf, result);
+            fgen(self, (int)nargs, smallbuf, result);
         } else {
             std::vector<void*> buf(nargs);
             runRelease = copy_args(args, nargs, buf.data());
-            faceptr.fGeneric(self, (int)nargs, buf.data(), result);
+            fgen(self, (int)nargs, buf.data(), result);
         }
         if (runRelease) release_args(args, nargs);
         return true;
@@ -970,15 +992,24 @@ bool Cppyy::IsEnum(const std::string& type_name)
     return gInterpreter->ClassInfo_IsEnum(tn_short.c_str());
 }
 
-bool Cppyy::IsAggregate(TCppType_t klass)
+bool Cppyy::IsAggregate(TCppType_t type)
 {
 // Test if this type is a "plain old data" type
-    TClassRef& cr = type_from_handle(klass);
+    TClassRef& cr = type_from_handle(type);
     if (cr.GetClass())
         return cr->ClassProperty() & kClassIsAggregate;
     return false;
 }
 
+bool Cppyy::IsDefaultConstructable(TCppType_t type)
+{
+// Test if this type has a default constructor or is a "plain old data" type
+    TClassRef& cr = type_from_handle(type);
+    if (cr.GetClass())
+        return cr->HasDefaultConstructor() || (cr->ClassProperty() & kClassIsAggregate);
+    return false;
+}
+
 // helpers for stripping scope names
 static
 std::string outer_with_template(const std::string& name)
@@ -1688,11 +1719,6 @@ Cppyy::TCppMethod_t Cppyy::GetMethodTemplate(
 // to do an explicit lookup that ignores the prototype (i.e. the full name should be
 // enough), and finally to ignore the template arguments part of the name as this fails
 // in cling if there are default parameters.
-// It would be possible to get the prototype from the created functions and use that to
-// do a new lookup, after which ROOT/meta will manage the function. However, neither
-// TFunction::GetPrototype() nor TFunction::GetSignature() is of the proper form, so
-// we'll/ manage the new TFunctions instead and will assume that they are cached on the
-// calling side to prevent multiple creations.
     TFunction* func = nullptr; ClassInfo_t* cl = nullptr;
     if (scope == (cppyy_scope_t)GLOBAL_HANDLE) {
         func = gROOT->GetGlobalFunctionWithPrototype(name.c_str(), proto.c_str());
@@ -1737,20 +1763,9 @@ Cppyy::TCppMethod_t Cppyy::GetMethodTemplate(
 
     if (func) {
     // make sure we didn't match a non-templated overload
-        if (strstr(func->GetName(), "<"))
+        if (func->ExtraProperty() & kIsTemplateSpec)
             return (TCppMethod_t)new_CallWrapper(func);
 
-    // don't give in just yet, but rather get the full name through the symbol name,
-    // as eg. constructors do not receive their proper/full name from GetName()
-        int err;
-        char* truename = TClassEdit::DemangleName(func->GetMangledName(), err);
-        if (truename && err != -1) {
-            bool isTemplated = (bool)strstr(truename, "<");
-            free(truename);
-            if (isTemplated)
-                return (TCppMethod_t)new_CallWrapper(func);
-        }
-
     // disregard this non-templated method as it will be considered when appropriate
         return (TCppMethod_t)nullptr;
     }
@@ -1778,7 +1793,7 @@ Cppyy::TCppMethod_t Cppyy::GetMethodTemplate(
 static inline
 std::string type_remap(const std::string& n1, const std::string& n2)
 {
-// Operator lookups of (C++ string, Python str) should succeeded, for the combos of
+// Operator lookups of (C++ string, Python str) should succeed for the combos of
 // string/str, wstring/str, string/unicode and wstring/unicode; since C++ does not have a
 // operator+(std::string, std::wstring), we'll have to look up the same type and rely on
 // the converters in CPyCppyy/_cppyy.
@@ -2179,6 +2194,10 @@ int cppyy_compile(const char* code) {
     return Cppyy::Compile(code);
 }
 
+int cppyy_compile_silent(const char* code) {
+    return Cppyy::Compile(code, true /* silent */);
+}
+
 char* cppyy_to_string(cppyy_type_t klass, cppyy_object_t obj) {
     return cppstring_to_cstring(Cppyy::ToString(klass, obj));
 }
@@ -2407,6 +2426,10 @@ int cppyy_is_aggregate(cppyy_type_t type) {
     return (int)Cppyy::IsAggregate(type);
 }
 
+int cppyy_is_default_constructable(cppyy_type_t type) {
+    return (int)Cppyy::IsDefaultConstructable(type);
+}
+
 const char** cppyy_get_all_cpp_names(cppyy_scope_t scope, size_t* count) {
     std::set<std::string> cppnames;
     Cppyy::GetAllCppNames(scope, cppnames);
