@@ -42,7 +42,7 @@ namespace Cppyy {
 
 // direct interpreter access -------------------------------------------------
     RPY_EXPORTED
-    bool Compile(const std::string& code);
+    bool Compile(const std::string& code, bool silent = false);
     RPY_EXPORTED
     std::string ToString(TCppType_t klass, TCppObject_t obj);
 
@@ -135,6 +135,8 @@ namespace Cppyy {
     bool IsEnum(const std::string& type_name);
     RPY_EXPORTED
     bool IsAggregate(TCppType_t type);
+    RPY_EXPORTED
+    bool IsDefaultConstructable(TCppType_t type);
 
     RPY_EXPORTED
     void GetAllCppNames(TCppScope_t scope, std::set<std::string>& cppnames);
